@@ -17,7 +17,7 @@ from .execute import (
 from .map_async_iterator import MapAsyncIterator
 from .subscribe import subscribe, create_source_event_stream
 from .middleware import MiddlewareManager
-from .values import get_directive_values, get_variable_values
+from .values import get_argument_values, get_directive_values, get_variable_values
 
 __all__ = [
     "create_source_event_stream",
@@ -32,6 +32,7 @@ __all__ = [
     "MapAsyncIterator",
     "Middleware",
     "MiddlewareManager",
+    "get_argument_values",
     "get_directive_values",
     "get_variable_values",
 ]
